package com.example.calculatrice

import android.content.Context
import android.os.Bundle
import android.widget.Button
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity

class MainActivity : AppCompatActivity() {

    private lateinit var display: TextView
    private var currentInput = ""
    private var lastOperator = ""
    private var lastNumber = 0.0
    private var isNewEntry = true

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        display = findViewById(R.id.display)

        loadLastCalculation()

        val buttons = listOf(
            R.id.btn0, R.id.btn1, R.id.btn2, R.id.btn3,
            R.id.btn4, R.id.btn5, R.id.btn6, R.id.btn7,
            R.id.btn8, R.id.btn9, R.id.btnDot,
            R.id.btnAdd, R.id.btnSubtract, R.id.btnMultiply, R.id.btnDivide,
            R.id.btnClear, R.id.btnClearEntry, R.id.btnEquals
        )


        for (id in buttons) {
            findViewById<Button>(id).setOnClickListener { onButtonClick(it as Button) }
        }
    }

    private fun onButtonClick(button: Button) {
        when (val text = button.text.toString()) {
            "C" -> clearAll()
            "CE" -> clearEntry()
            "+", "-", "*", "/" -> onOperator(text)
            "=" -> calculateResult()
            "." -> onDot()
            else -> onNumber(text)
        }
    }

    private fun onNumber(number: String) {
        if (isNewEntry) {
            currentInput = number
            isNewEntry = false
        } else {
            currentInput += number
        }
        updateDisplay()
    }

    private fun onDot() {
        if (!currentInput.contains(".")) {
            currentInput += "."
            updateDisplay()
        }
    }

    private fun onOperator(operator: String) {
        if (currentInput.isNotEmpty()) {
            lastNumber = currentInput.toDouble()
            lastOperator = operator
            isNewEntry = true
        }
    }

    private fun calculateResult() {
        if (currentInput.isNotEmpty() && lastOperator.isNotEmpty()) {
            val secondNumber = currentInput.toDouble()
            val result = when (lastOperator) {
                "+" -> lastNumber + secondNumber
                "-" -> lastNumber - secondNumber
                "*" -> lastNumber * secondNumber
                "/" -> if (secondNumber != 0.0) lastNumber / secondNumber else Double.NaN
                else -> 0.0
            }

            currentInput = result.toString()
            lastOperator = ""
            isNewEntry = true

            saveLastCalculation()
            updateDisplay()
        }
    }

    private fun clearAll() {
        currentInput = ""
        lastOperator = ""
        lastNumber = 0.0
        isNewEntry = true
        updateDisplay()
    }

    private fun clearEntry() {
        currentInput = ""
        isNewEntry = true
        updateDisplay()
    }

    private fun updateDisplay() {
        display.text = currentInput.ifEmpty { "0" }
    }

    private fun saveLastCalculation() {
        val sharedPreferences = getSharedPreferences("CalculatorPrefs", Context.MODE_PRIVATE)
        sharedPreferences.edit().putString("lastCalculation", currentInput).apply()
    }

    private fun loadLastCalculation() {
        val sharedPreferences = getSharedPreferences("CalculatorPrefs", Context.MODE_PRIVATE)
        currentInput = sharedPreferences.getString("lastCalculation", "") ?: ""
        updateDisplay()
    }
}
