# Lab 1 - Activities and Layouts

## Instructions 

- make regular and well-commented commits
- push your changes at the end of the lab

## New project creation

For all the following exercices:
- create a new project of the type "Empty Views Activity"
- choose the Kotlin language
- choose minimum API level 29
- keep the default build configuration language (Gradle Kotlin DSL) 


## 1. Getting started with Android Studio

### Project Configuration

- Create a new project of the type "Empty Views Activity":
    - Choose the Kotlin language
    - Choose minimum API level 29
    - Keep the default build configuration language (Gradle Kotlin DSL) 

### AVD Configuration
    
- Create a new AVD (Android Virtual Device):
    - Pick a phone device of your choice (for example Pixel 8 Pro)
    - Choose a system image with the latest API level (for example API 34 for Android 14). If the system image is not installed, click on the download icon.
    - Select a startup orientation (Portrait)

    And that's it! You are now ready to start developing your first Android application.
- Test the emulator by running your AVD image. You should see the Android home screen.

### First application: activity lifecycle

Write an application composed of a single activity that displays a `Toast` message and a message in the log using `Log.d(...)` with the current state at each state change for the following states:

- `onCreate`
- `onStart`
- `onResume`
- `onPause`
- `onStop`
- `onDestroy`


Run your application and observe the logs by rotating the screen, pressing the "Home" button, and pressing the "Back" button.


## 2. Linear layout

Write an application composed of a single activity that displays the layout illustrated below. It is composed of `TextView`, `ImageView`, `EditText`, `Button` elements.

To do this, create a XML layout file using only a combination of `LinearLayout`.


When the "Validate" button is clicked, the text of the bottom `TextView` is updated with the content of the `EditText`.
When the "Clear" button is clicked, the content of the `EditText` and bottom `TextView` is cleared.


1. What happens when the orientation of the screen changes?
2. Using the method `onSaveInstanceState`, save the content of the bottom `TextView` when the activity is destroyed and restore it when the activity is recreated.

<img src="img/Lab1.2.png" width="300"  >

Some hints:
- put the image `img/android.jpg` in the `res/drawable` folder and reference it in the `ImageView` element
- to add icons on the buttons:
    - first add vector assets by double clicking on the `res` folder 
    - click on the `clip art` button and search for the desired icon
    - use the property `leftDrawable`  to add the icon to the button
    - you can define custom colors in the `res/values/colors.xml` file
    - you can use the `android:hint` property to add a placeholder text in the `EditText` and `TextView` elements


## 3. Advanced layout: calculator

Create an application composed of a single activity that simulates a simple calculator. The calculator must be able to perform the following operations: addition, subtraction, multiplication, and division. The calculator must be able to manage the following keys: 0 to 9, dot (.), +, -, *, /, =, C (clear), and CE (clear entry).

The layout of the calculator must be created in an XML file, using a `ConstraintLayout`.
You can use a `GridLayout` to display the buttons.

<img src="img/calculator.png" width="300"  >

Functionalities:

- When a digit is pressed, it is concatenated with the current number displayed
- When an operation is selected (like +, -, *, /), it should calculate the result up to that point and display it.
- When the equal sign is pressed, the final result is displayed.
- When the clear button is pressed, the display is cleared and the current operation is reset
- When the clear entry button is pressed, it clears the displayed number but keeps the current operation
- When the dot button is pressed, it adds a dot to the current number, if it is not already present 

1. Run and test your application
2. Save the last calculation in `SharedPreferences`: the calculation is restored when the application is restarted
