package com.example.communication_application

import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Patterns
import android.widget.Button
import android.widget.EditText
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity

class MainActivity : AppCompatActivity() {

    private lateinit var sharedPreferences: SharedPreferences
    private lateinit var editName: EditText
    private lateinit var editFirstName: EditText
    private lateinit var editPhone: EditText
    private lateinit var editEmail: EditText
    private lateinit var btnClear: Button
    private lateinit var btnSend: Button

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        // Initialisation de SharedPreferences
        sharedPreferences = getSharedPreferences("UserInfo", Context.MODE_PRIVATE)

        // Initialisation des vues
        editName = findViewById(R.id.editName)
        editFirstName = findViewById(R.id.editFirstName)
        editPhone = findViewById(R.id.editPhone)
        editEmail = findViewById(R.id.editEmail)
        btnClear = findViewById(R.id.btnClear)
        btnSend = findViewById(R.id.btnSend)

        // Charger les données enregistrées
        loadSavedData()

        btnClear.setOnClickListener {
            clearFields()
        }

        btnSend.setOnClickListener {
            if (validateFields()) {
                saveData()
                sendDataToSecondActivity()
            }
        }
    }

    private fun validateFields(): Boolean {
        val name = editName.text.toString().trim()
        val firstName = editFirstName.text.toString().trim()
        val phone = editPhone.text.toString().trim()
        val email = editEmail.text.toString().trim()

        return when {
            name.isEmpty() || firstName.isEmpty() || phone.isEmpty() || email.isEmpty() -> {
                Toast.makeText(this, getString(R.string.fill_all_fields), Toast.LENGTH_SHORT).show()
                false
            }
            !Patterns.PHONE.matcher(phone).matches() -> {
                Toast.makeText(this, "Numéro de téléphone invalide", Toast.LENGTH_SHORT).show()
                false
            }
            !Patterns.EMAIL_ADDRESS.matcher(email).matches() -> {
                Toast.makeText(this, "Adresse e-mail invalide", Toast.LENGTH_SHORT).show()
                false
            }
            else -> true
        }
    }

    private fun saveData() {
        with(sharedPreferences.edit()) {
            putString("name", editName.text.toString())
            putString("firstName", editFirstName.text.toString())
            putString("phone", editPhone.text.toString())
            putString("email", editEmail.text.toString())
            apply()
        }
    }

    private fun loadSavedData() {
        editName.setText(sharedPreferences.getString("name", ""))
        editFirstName.setText(sharedPreferences.getString("firstName", ""))
        editPhone.setText(sharedPreferences.getString("phone", ""))
        editEmail.setText(sharedPreferences.getString("email", ""))
    }

    private fun clearFields() {
        editName.text.clear()
        editFirstName.text.clear()
        editPhone.text.clear()
        editEmail.text.clear()
    }

    private fun sendDataToSecondActivity() {
        val intent = Intent(this, SecondActivity::class.java).apply {
            putExtra("name", editName.text.toString())
            putExtra("firstName", editFirstName.text.toString())
            putExtra("phone", editPhone.text.toString())
            putExtra("email", editEmail.text.toString())
        }
        startActivity(intent)
    }
}
