package com.example.contactrecyclerappmodel

import android.content.Intent
import android.os.Bundle
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.example.contactrecyclerappmodel.adapter.ContactAdapter
import com.example.contactrecyclerappmodel.model.Contact
import com.google.android.material.floatingactionbutton.FloatingActionButton

class MainActivity : AppCompatActivity() {

    private lateinit var recyclerView: RecyclerView
    private lateinit var contactAdapter: ContactAdapter
    private val contactList = mutableListOf<Contact>()
    private var nextId = 1

    private val addContactLauncher = registerForActivityResult(
        ActivityResultContracts.StartActivityForResult()
    ) { result ->
        if (result.resultCode == RESULT_OK) {
            val data = result.data
            if (data != null) {
                val name = data.getStringExtra("name") ?: ""
                val address = data.getStringExtra("address") ?: ""
                val phone = data.getStringExtra("phone") ?: ""
                val email = data.getStringExtra("email") ?: ""
                val image = if (nextId % 2 == 0) R.drawable.profile1 else R.drawable.profile2
                val contact = Contact(nextId++, name, address, phone, email, image)
                contactList.add(contact)
                contactAdapter.notifyDataSetChanged()
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        recyclerView = findViewById(R.id.recyclerView)
        recyclerView.layoutManager = LinearLayoutManager(this)

        generateSampleContacts()

        contactAdapter = ContactAdapter(this, contactList) { contactToDelete ->
            contactList.remove(contactToDelete)
        }

        recyclerView.adapter = contactAdapter

        findViewById<FloatingActionButton>(R.id.fab).setOnClickListener {
            val intent = Intent(this, AddContactActivity::class.java)
            addContactLauncher.launch(intent)
        }
    }

    private fun generateSampleContacts() {
        val names = listOf("Lucas Durand", "Jean Richard", "Lucas Martin", "Jean Martin", "Antoine Richard", "Jean Robert")
        val addresses = listOf("60 avenue de la République, Marseille", "97 rue des Ecoles, Marseille", "75 rue des Ecoles, Nice", "94 boulevard Saint-Germain, Nice", "29 place de la Concorde, Montpellier", "93 rue Pasteur, Lyon")
        val phones = listOf("0564390626", "0543028644", "0589700101", "0516821262", "0516663676", "0519457508")
        val emails = listOf("l.durand@hotmail.com", "j.richard@unistra.fr", "l.martin@unistra.fr", "j.martin@outlook.fr", "a.richard@yahoo.fr", "j.robert@laposte.net")

        for (i in names.indices) {
            val image = if (i % 2 == 0) R.drawable.profile1 else R.drawable.profile2
            contactList.add(Contact(nextId++, names[i], addresses[i], phones[i], emails[i], image))
        }
    }
}
