package com.example.contactrecyclerappmodel.adapter

import android.app.AlertDialog
import android.content.Context
import android.graphics.Color
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.example.contactrecyclerappmodel.R
import com.example.contactrecyclerappmodel.model.Contact
import com.example.contactrecyclerappmodel.viewholder.ContactViewHolder

class ContactAdapter(
    private val context: Context,
    private val contactList: MutableList<Contact>,
    private val onDelete: (Contact) -> Unit
) : RecyclerView.Adapter<ContactViewHolder>() {

    private var selectedPosition = -1

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ContactViewHolder {
        val view = LayoutInflater.from(parent.context).inflate(R.layout.contact_item, parent, false)
        return ContactViewHolder(view)
    }

    override fun onBindViewHolder(holder: ContactViewHolder, position: Int) {
        val contact = contactList[position]
        holder.name_tv.text = contact.name
        holder.address_tv.text = contact.address
        holder.phone_tv.text = contact.phone
        holder.email_tv.text = contact.email
        holder.image_iv.setImageResource(contact.imageResId)

        holder.itemView.setBackgroundColor(
            if (selectedPosition == position) Color.LTGRAY else Color.TRANSPARENT
        )

        holder.itemView.setOnClickListener {
            val oldPos = selectedPosition
            selectedPosition = position
            notifyItemChanged(oldPos)
            notifyItemChanged(position)
        }

        holder.itemView.setOnLongClickListener {
            AlertDialog.Builder(holder.itemView.context)
                .setTitle("Delete Contact")
                .setMessage("Do you want to delete ${contact.name}?")
                .setPositiveButton("Yes") { _, _ ->
                    onDelete(contact)
                    notifyDataSetChanged()
                }
                .setNegativeButton("No", null)
                .show()
            true
        }
    }

    override fun getItemCount() = contactList.size
}
