# Lab 3 - RecyclerView

## Instructions

- Make regular and well-commented commits as needed
- Push at the end of the session

## RecyclerView

The `RecyclerView`  is used to display a list of items in a scrollable way. The `RecyclerView` is more efficient than the `ListView` because it reuses the views that are no longer visible to the user. The `RecyclerView` is composed of the following elements:

- `RecyclerView`: the main view that contains the list of items
- `Adapter`: the class that binds the data to the view
- `ViewHolder`: the class that holds the view of an item
- `LayoutManager`: the class that manages the layout of the items

For more details on `RecyclerView`, check the course and read the official documentation: https://developer.android.com/guide/topics/ui/layout/recyclerview


## Contacts

In this lab we will create a simple contact list application using a `RecyclerView`.

This app is composed of a single activity that displays a list of contacts in a `RecyclerView`.
A new contact can be inserted using a floating action button.
When a contact is clicked, the item will change its background color.
When a long click is performed on a contact, a dialog will be displayed to delete the item.

### Layout

The layout of the main activity is composed of a title (`TextView`) and a `RecyclerView` (taking the remaining space).

### Data 

Create a data class `Contact` that represents a contact. The class should contain the following fields:
- `id`: the unique identifier of the contact
- `name`: the name of the contact
-  `address`: the address of the contact
- `phone`: the phone number of the contact
- `email`: the email address of the contact
- `image`: an image representing the contact 
### Layout of the contact item

Define the layout of a single contact element in a XML file `contact_item.xml`.
You should use a `ConstraintLayout` containing the following elements:
- a `TextView` to display the name of the contact
- a `TextView` to display the phone number of the contact
- a `TextView` to display the address of the contact
- a `TextView` to display the email address of the contact
- an `ImageView` to display the image of the contact

The layout should look like this (but feel free to customize it):

<img src="img/contact_item.png"  width=300/>

### Viewholder

In the file `ContactViewHolder.kt`, implement the class  `ContactViewHolder` which extends `ViewHolder`. The viewholder will hold the references of the views of a contact item:

- var image_iv: ImageView 
- var name_tv: TextView 
- var phone_tv: TextView 
- var address_tv: TextView 
- var email_tv: TextView 

The constructor of the viewholder takes a `View` as parameter and retrieve the references of the views using the method `findViewById`.

### Adapter

In the file `ContactAdapter.kt`, implement the class `ContactAdapter` which extends `Adapter<ContactViewHolder>`. The adapter will bind the data to the view. 
The constructor of `ContactAdapter` takes a list of contacts, `contactList`.
The adapter must overrides the following methods:
- `onCreateViewHolder`: 
    - Obtain a `LayoutInflater` from the parent (using `from()` method)
    - Inflate the layout `contact_item.xml` using the method `LayoutInflater.inflate` 
    - Return a new `ContactViewHolder` with the inflated view
- `onBindViewHolder`: bind the data to the view. The method takes a `ContactViewHolder` and a position as parameters. The method should:
    - Retrieve a contact at the given position of the list `contactList`
    - Bind the data to the viewholder: set the name, phone, address, email of the contact to the corresponding views of the viewholder
    - For now, you can set a default image to the `ImageView`. We will see later how to load an image from an URL.
- `getItemCount`: return the number of items in the list.

### Main activity

The main activity will create a list of contacts and display it in a `RecyclerView`. The list of contacts should be initialized with some  data: create a list of at least 10 contacts with generic names, addresses, phone numbers, and email addresses. You can also rely on ChatGPT to generate a bunch of dummy contacts.

In the main activity, you should:
- retrieve a reference to the `RecyclerView` from the layout using the method `findViewById`
- create a new `ContactAdapter` with the list of contacts
- set the adapter to the `RecyclerView`
- set a `LinearLayoutManager` to the `RecyclerView` to manage the layout of the items using a vertical orientation

Finally, you should obtain the following result and be able to scroll through the list of contacts:

<img src="img/result.png"  width=300/>


### Add an image to the contact item

To add an image to the contact item, you can use the library `Glide` to load an image from an URL.
For example, you can use this URL  https://avatar.iran.liara.run/public/ and append it a number between 1 and 100 to get an avatar.

To use `Glide` in your project you need to follow these steps:
1.  Add the following line to your `build.gradle` file in the `dependencies` section:

    ```gradle
    implementation(libs.glide)
    ```

2. Import the library in your activity:
    ```kotlin
    import com.bumptech.glide.Glide
    ```

3. Add internet permission to your `AndroidManifest.xml` file:
    ```xml
    <uses-permission android:name="android.permission.INTERNET"/>
    ```

4. Load the image in the `onBindViewHolder` method of the adapter:
    ```kotlin
    Glide.with(context)
        .load("https://avatar.iran.liara.run/public/"+id)
        .into(holder.image_iv)
    ```
    where `id` should be a number between 1 and 100: you can use the `id` of the contact for example modulo 100.

### Add a floating action button

Add a floating action button to the main activity to insert a new contact. When the button is clicked, a new activity  `AddContactActivity` is launched. The activity contains a form to insert a new contact with the following fields:
- name
- address
- phone
- email

The activity contains a button to save the contact. 
When the button is clicked, the activity returns the content of the fields to the main activity, using the Result API:
https://developer.android.com/training/basics/intents/result

The main activity  adds the new contact to the list and updates the `RecyclerView`: the new contact should be displayed in the list.

### Click and long click

Add a click and long click listener to the contact item:
- When a contact is clicked, the item should change its background color. 
- When a long click is performed on a contact, an alert dialog is displayed to confirm the deletion of the item.

Hints:
- To change the background color of the item, you can use the method `setBackgroundColor`.
- To memorize the position of the item clicked, you can store it in a variable of the adapter.
- To display an alert dialog, you can use the `AlertDialog.Builder` class. 
See: https://developer.android.com/develop/ui/views/components/dialogs
- To notify the recyclerview that the data has changed and update the view, you have to call the method `notifyDataSetChanged` of the adapter.

### Bonus

Add a drag-and-drop feature enabling to reorder the items in the list.